@@ -57,7 +57,7 @@ impl LocalExt for Local {
             Err(crev_lib::Error::GitUrlNotConfigured) => {
                 bail!("Id has no public URL set. Run `cargo crev id set-url <your-public-git-proof-repo>` and try again. See https://github.com/crev-dev/cargo-crev/discussions for help.");
             }
-            Err(e) => Err(e)?,
+            Err(e) => Err(e.into()),
         }
     }
 }
@@ -802,7 +802,7 @@ fn set_trust_level_for_ids(
 
         db.get_trust_proof_between(&unlocked_id.id.id, &ids[0])
             .map(|trust_proof| trust_proof.override_.clone())
-            .unwrap_or(vec![])
+            .unwrap_or_default()
     } else {
         vec![]
     };
@@ -827,10 +827,9 @@ fn set_trust_level_for_ids(
 
             if show_override_suggestions {
                 let db = local.load_db()?;
-                for (id, trust_level) in ids.into_iter().flat_map(|id| db.get_reverse_trust_for(id))
-                {
+                for (id, trust_level) in ids.iter().flat_map(|id| db.get_reverse_trust_for(id)) {
                     let (status, url) = url_to_status_str(&db.lookup_url(id));
-                    writeln!(text, "# - id-type: {}", "crev")?; // TODO: support other ids?
+                    writeln!(text, "# - id-type: crev")?; // TODO: support other ids?
                     writeln!(text, "#   id: {} # level: {}", id, trust_level)?;
                     writeln!(text, "#   url: {} # {}", url, status)?;
                     writeln!(text, "#   comment: \"\"")?;
