@@ -1,7 +1,10 @@
 use anyhow::{bail, Result};
 use crev_data::{Level, Version};
-use std::{ffi::OsString, path::PathBuf};
+use std::{ffi::OsString, io::Write, path::PathBuf};
 use structopt::StructOpt;
+use term::color;
+
+use crate::term::Term;
 
 #[derive(Debug, StructOpt, Clone, Default)]
 pub struct CrateSelector {
@@ -38,9 +41,28 @@ impl CrateSelector {
             (None, None) => Ok(None),
         }
     }
-}
 
-impl CrateSelector {
+    /// If can't find manifest file in CWD, change to `-u`
+    ///
+    /// This is so some commands can conveniently work outside of any
+    /// Rust project.
+    pub fn auto_unrelated(self) -> Result<Self> {
+        let repo = crate::Repo::auto_open_cwd_default()?;
+
+        Ok(match repo.get_manifest_path() {
+            Ok(_) => self,
+            Err(e) => {
+                Term::new().eprintln(
+                    format_args!("Unable to find manifest: {e}. Assuming `-u` option."),
+                    Some(color::YELLOW),
+                )?;
+                CrateSelector {
+                    unrelated: true,
+                    ..self
+                }
+            }
+        })
+    }
     pub fn is_empty(&self) -> bool {
         self.name.is_none() && self.version.is_none()
     }
@@ -59,19 +81,27 @@ pub struct CargoOpts {
     #[structopt(long = "features", value_name = "FEATURES")]
     /// [cargo] Space-separated list of features to activate
     pub features: Option<String>,
+
     #[structopt(long = "all-features")]
     /// [cargo] Activate all available features
     pub all_features: bool,
+
     #[structopt(long = "no-default-features")]
     /// [cargo] Do not activate the `default` feature
     pub no_default_features: bool,
+
+    #[structopt(long = "dev-dependencies")]
+    /// [cargo] Skip dev dependencies.
+    dev_dependencies: bool,
+
     #[structopt(long = "no-dev-dependencies")]
     /// [cargo] Skip dev dependencies.
-    pub no_dev_dependencies: bool,
-    #[structopt(long = "manifest-path", value_name = "PATH", parse(from_os_str))]
+    no_dev_dependencies: bool,
 
+    #[structopt(long = "manifest-path", value_name = "PATH", parse(from_os_str))]
     /// [cargo] Path to Cargo.toml
     pub manifest_path: Option<PathBuf>,
+
     #[structopt(short = "Z", value_name = "FLAG")]
 
     /// [cargo] Unstable (nightly-only) flags to Cargo
@@ -83,6 +113,19 @@ pub struct CargoOpts {
     pub target: Option<Option<String>>,
 }
 
+impl CargoOpts {
+    pub fn dev_dependencies(&self) -> Result<bool> {
+        if self.dev_dependencies && self.no_dev_dependencies {
+            bail!("`--no-dev-dependencies` and `--dev-dependencies` can't be used together");
+        }
+        if self.no_dev_dependencies {
+            writeln!(std::io::stderr(), "`--no-dev-dependencies` is the default, is now ignored and will be removed in the future")?;
+        }
+
+        Ok(self.dev_dependencies)
+    }
+}
+
 #[derive(Debug, StructOpt, Clone)]
 pub struct IdNew {
     #[structopt(long = "url")]
@@ -105,7 +148,7 @@ pub struct IdSwitch {
 /// Parameters describing trust graph traversal
 #[derive(Debug, StructOpt, Clone, Default)]
 pub struct TrustDistanceParams {
-    #[structopt(long = "depth", default_value = "10")]
+    #[structopt(long = "depth", default_value = "20")]
     /// [trust-graph-traversal] Maximum allowed distance from the root identity when traversing trust graph
     pub depth: u64,
 
@@ -197,16 +240,19 @@ pub struct Update {
 }
 
 #[derive(Debug, StructOpt, Clone, Default)]
-pub struct CrateVerifyCommon {
+pub struct WotOpts {
     #[structopt(flatten)]
     pub trust_params: TrustDistanceParams,
 
-    #[structopt(flatten)]
-    pub requirements: VerificationRequirements,
-
     #[structopt(long = "for-id")]
     /// Root identity to calculate the Web of Trust for [default: current user id]
     pub for_id: Option<String>,
+}
+
+#[derive(Debug, StructOpt, Clone, Default)]
+pub struct CrateVerifyCommon {
+    #[structopt(flatten)]
+    pub requirements: VerificationRequirements,
 
     #[structopt(flatten)]
     pub cargo_opts: CargoOpts,
@@ -329,6 +375,9 @@ pub struct CrateVerify {
     #[structopt(flatten)]
     pub common: CrateVerifyCommon,
 
+    #[structopt(flatten)]
+    pub wot: WotOpts,
+
     #[structopt(flatten)]
     pub columns: CrateVerifyColumns,
 
@@ -354,6 +403,10 @@ pub struct CrateVerify {
 
 #[derive(Debug, StructOpt, Clone)]
 pub struct IdTrust {
+    #[structopt(long = "overrides")]
+    /// Enable overrides suggestions
+    pub overrides: bool,
+
     /// Public IDs to create Trust Proof for
     pub public_ids: Vec<String>,
 
@@ -367,6 +420,10 @@ pub struct IdTrust {
 
 #[derive(Debug, StructOpt, Clone)]
 pub struct TrustUrls {
+    #[structopt(long = "overrides")]
+    /// Enable overrides suggestions
+    pub overrides: bool,
+
     /// Public IDs or proof repo URLs to create Trust Proof for
     pub public_ids_or_urls: Vec<String>,
 
@@ -575,6 +632,10 @@ pub struct CrateReview {
     #[structopt(long = "skip-activity-check")]
     pub skip_activity_check: bool,
 
+    #[structopt(long = "overrides")]
+    /// Enable overrides suggestions
+    pub overrides: bool,
+
     /// Review the delta since the given version
     #[structopt(long = "diff", name = "base-version")]
     #[allow(clippy::option_option)]
@@ -672,6 +733,15 @@ pub enum Id {
     Query(IdQuery),
 }
 
+#[derive(Debug, StructOpt, Clone)]
+pub struct CrateVerifyFull {
+    #[structopt(flatten)]
+    pub opts: CrateVerify,
+
+    #[structopt(flatten)]
+    pub crate_: CrateSelector,
+}
+
 #[derive(Debug, StructOpt, Clone)]
 pub enum Crate {
     /// Start a shell in source directory of a crate under review
@@ -682,6 +752,11 @@ pub enum Crate {
     #[structopt(name = "open")]
     Open(CrateOpen),
 
+    /// WIP: Expand the crate source using `cargo-expand` like functionality
+    // https://github.com/dtolnay/cargo-expand/issues/11
+    #[structopt(name = "expand")]
+    Expand(ReviewOrGotoCommon),
+
     /// Clean the source code directory of a crate (eg. after review)
     #[structopt(name = "clean")]
     Clean(ReviewOrGotoCommon),
@@ -698,19 +773,17 @@ pub enum Crate {
 
     /// Verify dependencies
     #[structopt(name = "verify")]
-    Verify {
-        #[structopt(flatten)]
-        opts: CrateVerify,
-
-        #[structopt(flatten)]
-        crate_: CrateSelector,
-    },
+    Verify(CrateVerifyFull),
 
     /// Most valuable players (reviewers)
     #[structopt(name = "mvp")]
     Mvp {
         #[structopt(flatten)]
         opts: CrateVerifyCommon,
+
+        #[structopt(flatten)]
+        wot: WotOpts,
+
         #[structopt(flatten)]
         crate_: CrateSelector,
     },
@@ -732,6 +805,10 @@ pub enum Crate {
     Info {
         #[structopt(flatten)]
         opts: CrateVerifyCommon,
+
+        #[structopt(flatten)]
+        wot: WotOpts,
+
         #[structopt(flatten)]
         crate_: CrateSelector,
     },
@@ -753,6 +830,14 @@ pub enum Config {
     /// Print the dir containing config files
     #[structopt(name = "dir")]
     Dir,
+
+    /// Print the dir containing data files
+    #[structopt(name = "data-dir")]
+    DataDir,
+
+    /// Print the dir containing cache files
+    #[structopt(name = "cache-dir")]
+    CacheDir,
 }
 
 #[derive(Debug, StructOpt, Clone)]
@@ -819,6 +904,16 @@ pub enum Proof {
     Find(ProofFind),
 }
 
+#[derive(Debug, StructOpt, Clone)]
+pub enum Wot {
+    /// Create a new Id
+    #[structopt(name = "log")]
+    Log {
+        #[structopt(flatten)]
+        wot: WotOpts,
+    },
+}
+
 #[derive(Debug, StructOpt, Clone)]
 #[structopt(setting = structopt::clap::AppSettings::DeriveDisplayOrder)]
 #[structopt(setting = structopt::clap::AppSettings::DisableHelpSubcommand)]
@@ -847,6 +942,10 @@ pub enum Command {
     /// Add a Trust proof by an Id or a URL
     Trust(TrustUrls),
 
+    /// Web of Trust
+    #[structopt(name = "wot")]
+    Wot(Wot),
+
     /// Shortcut for `crate goto`
     #[structopt(name = "goto")]
     Goto(ReviewOrGotoCommon),
@@ -869,13 +968,7 @@ pub enum Command {
 
     /// Shortcut for `crate verify`
     #[structopt(name = "verify")]
-    Verify {
-        #[structopt(flatten)]
-        opts: CrateVerify,
-
-        #[structopt(flatten)]
-        crate_: CrateSelector,
-    },
+    Verify(CrateVerifyFull),
 }
 
 /// Cargo will pass the name of the `cargo-<tool>`
@@ -885,8 +978,8 @@ pub enum MainCommand {
     #[structopt(name = "crev")]
     #[structopt(after_help = r#"All commands can be abbreviated.
 
-Join Matrix channel for more help at https://matrix.to/#/#crev:matrix.org
-Read user documentation at https://docs.rs/crate/cargo-crev
+Help and feedback: https://github.com/crev-dev/cargo-crev/discussions/
+User documentation: https://docs.rs/crate/cargo-crev
         "#)]
     Crev(Command),
 }
