@@ -6,12 +6,15 @@ use cargo::{
         manifest::ManifestMetadata,
         package::PackageSet,
         registry::PackageRegistry,
-        resolver::{features::HasDevUnits, CliFeatures},
+        resolver::{features::RequestedFeatures, CliFeatures, ResolveOpts, VersionPreferences},
         source::SourceMap,
-        FeatureValue, Package, PackageId, PackageIdSpec, Resolve, SourceId, Workspace,
+        Package, PackageId, Resolve, SourceId, Workspace,
+    },
+    util::{
+        config::{Config, ConfigValue},
+        important_paths::find_root_manifest_for_wd,
+        CargoResult, Rustc,
     },
-    ops,
-    util::{important_paths::find_root_manifest_for_wd, CargoResult, Rustc},
 };
 use cargo_platform::Cfg;
 use petgraph::graph::NodeIndex;
@@ -19,7 +22,6 @@ use std::{
     collections::{hash_map::Entry, BTreeSet, HashMap, HashSet},
     env,
     path::PathBuf,
-    rc::Rc,
     str::{self, FromStr},
 };
 
@@ -39,14 +41,11 @@ pub struct Graph {
 }
 
 impl Graph {
-    pub fn get_all_pkg_ids<'s>(&'s self) -> impl Iterator<Item = PackageId> + 's {
+    pub fn get_all_pkg_ids(&self) -> impl Iterator<Item = PackageId> + '_ {
         self.nodes.keys().cloned()
     }
 
-    pub fn get_dependencies_of<'s>(
-        &'s self,
-        pkg_id: PackageId,
-    ) -> impl Iterator<Item = PackageId> + 's {
+    pub fn get_dependencies_of(&self, pkg_id: PackageId) -> impl Iterator<Item = PackageId> + '_ {
         self.nodes
             .get(&pkg_id)
             .into_iter()
@@ -57,10 +56,10 @@ impl Graph {
             .map(move |node_idx| self.graph.node_weight(node_idx).unwrap().id)
     }
 
-    pub fn get_reverse_dependencies_of<'s>(
-        &'s self,
+    pub fn get_reverse_dependencies_of(
+        &self,
         pkg_id: PackageId,
-    ) -> impl Iterator<Item = PackageId> + 's {
+    ) -> impl Iterator<Item = PackageId> + '_ {
         self.nodes
             .get(&pkg_id)
             .into_iter()
@@ -126,12 +125,12 @@ fn get_cfgs(rustc: &Rustc, target: Option<&str>) -> Result<Vec<Cfg>> {
 }
 
 fn our_resolve<'a, 'cfg>(
-    registry: &mut PackageRegistry<'cfg>,
+    mut registry: PackageRegistry<'cfg>,
     workspace: &'a Workspace<'cfg>,
     features: &[String],
     all_features: bool,
     no_default_features: bool,
-    no_dev_dependencies: bool,
+    dev_dependencies: bool,
 ) -> CargoResult<(PackageSet<'cfg>, Resolve)> {
     // there is bunch of slightly different ways to do it,
     // so I leave some dead code around, in case I want to
@@ -140,6 +139,40 @@ fn our_resolve<'a, 'cfg>(
     // this one will create a `Cargo.lock` file if it didn't exist before
     // good? not good? it also uses the registry to make it possible
     // the other methods
+    // let mut registry = PackageRegistry::new(&workspace.config())?;
+    let _lock = workspace.config().acquire_package_cache_lock()?;
+    registry.lock_patches();
+    let summaries: Vec<_> = workspace
+        .members()
+        .map(|m| {
+            Ok((
+                m.summary().to_owned(),
+                ResolveOpts {
+                    dev_deps: dev_dependencies,
+                    features: RequestedFeatures::CliFeatures(CliFeatures::from_command_line(
+                        features,
+                        all_features,
+                        !no_default_features,
+                    )?),
+                },
+            ))
+        })
+        .collect::<Result<Vec<_>>>()?;
+    let resolve = cargo::core::resolver::resolve(
+        &summaries,
+        workspace.root_replace(),
+        &mut registry,
+        &VersionPreferences::default(),
+        None,
+        false,
+    )?;
+
+    Ok((
+        cargo::ops::get_resolved_packages(&resolve, registry)?,
+        resolve,
+    ))
+
+    /* no_default_features doesn't work in this one
     let (packages, resolve) = ops::resolve_ws(workspace)?;
 
     let cli_features = CliFeatures {
@@ -175,6 +208,7 @@ fn our_resolve<'a, 'cfg>(
     )?;
 
     Ok((packages, resolve))
+        */
 
     /*
     // this method does not allow passing no_dev_dependencies
@@ -270,10 +304,69 @@ fn build_graph<'a>(
     Ok(graph)
 }
 
+/// Modifies the given config so that directory source replacements are removed, and references to them as well.
+///
+/// - For information on directory sources, [see here](https://doc.rust-lang.org/cargo/reference/source-replacement.html#directory-sources)
+/// - For information on source replacement, [see here](https://doc.rust-lang.org/cargo/reference/config.html#source)
+fn prune_directory_source_replacements(
+    config: &mut HashMap<String, ConfigValue>,
+) -> CargoResult<()> {
+    if let Some(ConfigValue::Table(ref mut source_config, _)) = config.get_mut("source") {
+        // To do the pruning, first, generate a graph of registry sources, where the node are the sources, and there is an edge if a source
+        //  defines that it is replaced with another source.
+        // Then, find the directory sources, and traverse the graph in reverse to find all the sources that are directory sources, or reference them
+        //  directly or indirectly.
+        // Then, the found sources can be removed from the config.
+        let mut source_graph = petgraph::Graph::<String, ()>::new();
+        let nodes = source_config
+            .keys()
+            .map(|source_key| (source_key, source_graph.add_node(source_key.clone())))
+            .collect::<HashMap<_, _>>();
+
+        source_config
+            .iter()
+            .for_each(|(source_name, source_config_entry)| {
+                if let ConfigValue::Table(source_config_entry, _) = source_config_entry {
+                    if let Some(ConfigValue::String(replacement, _)) =
+                        source_config_entry.get("replace-with")
+                    {
+                        source_graph.add_edge(
+                            *nodes.get(source_name).unwrap(),
+                            *nodes.get(replacement).unwrap(),
+                            (),
+                        );
+                    }
+                }
+            });
+        source_graph.reverse();
+        let source_entries_to_delete: HashSet<&String> = source_graph
+            .externals(petgraph::Direction::Incoming)
+            .filter(|leaf_node| {
+                let leaf_source = &source_graph[*leaf_node];
+                if let ConfigValue::Table(ref source_config_entry, _) = source_config[leaf_source] {
+                    if let Some(ConfigValue::String(_, _)) = source_config_entry.get("directory") {
+                        return true;
+                    }
+                }
+                false
+            })
+            .flat_map(|leaf_node| {
+                petgraph::visit::Walker::iter(
+                    petgraph::visit::Bfs::new(&source_graph, leaf_node),
+                    &source_graph,
+                )
+            })
+            .map(|node| &source_graph[node])
+            .collect();
+
+        source_config.retain(|source_name, _| !source_entries_to_delete.contains(source_name));
+    }
+    Ok(())
+}
+
 /// A handle to the current Rust project
 pub struct Repo {
-    manifest_path: PathBuf,
-    config: cargo::util::config::Config,
+    config: Config,
     cargo_opts: opts::CargoOpts,
     features_list: Vec<String>,
 }
@@ -283,14 +376,17 @@ impl Repo {
         Self::auto_open_cwd(Default::default())
     }
 
-    pub fn auto_open_cwd(cargo_opts: opts::CargoOpts) -> Result<Self> {
-        let manifest_path = if let Some(ref path) = cargo_opts.manifest_path {
+    pub fn get_manifest_path(&self) -> Result<PathBuf> {
+        Ok(if let Some(ref path) = self.cargo_opts.manifest_path {
             path.to_owned()
         } else {
             let cwd = env::current_dir()?;
             find_root_manifest_for_wd(&cwd)?
-        };
-        let mut config = cargo::util::config::Config::default()?;
+        })
+    }
+
+    pub fn auto_open_cwd(cargo_opts: opts::CargoOpts) -> Result<Self> {
+        let mut config = Config::default()?;
         config.configure(
             0,
             /* quiet */ false,
@@ -302,6 +398,10 @@ impl Repo {
             &cargo_opts.unstable_flags,
             &[],
         )?;
+
+        config.load_values()?;
+        prune_directory_source_replacements(config.values_mut()?)?;
+
         // how it used to be; can't find it anywhere anymore
         // let features_set =
         //     Method::split_features(&[cargo_opts.features.clone().unwrap_or_else(String::new)]);
@@ -309,22 +409,21 @@ impl Repo {
         let features_list = cargo_opts
             .features
             .clone()
-            .unwrap_or_else(String::new)
+            .unwrap_or_default()
             .split(',')
             .map(String::from)
             .filter(|s| !s.is_empty())
             .collect();
 
         Ok(Repo {
-            manifest_path,
             config,
             features_list,
             cargo_opts,
         })
     }
 
-    fn workspace(&self) -> CargoResult<Workspace<'_>> {
-        Workspace::new(&self.manifest_path, &self.config)
+    fn workspace(&self) -> Result<Workspace<'_>> {
+        Workspace::new(&self.get_manifest_path()?, &self.config)
     }
 
     // TODO: Do we even need it? We should just always use a default/empty
@@ -339,30 +438,36 @@ impl Repo {
         Ok(registry)
     }
 
-    pub fn get_dependency_graph(&self, roots: Vec<PackageId>) -> CargoResult<Graph> {
+    fn get_registry_from_workspace_members(&self) -> Result<(Workspace, PackageRegistry<'_>)> {
         let workspace = self.workspace()?;
-
-        let mut registry = self.registry(
+        let registry = self.registry(
             workspace
                 .members()
                 .map(|m| m.summary().source_id().to_owned()),
         )?;
-        // let root_sources: Vec<_> = roots.iter().map(|p| p.source_id().to_owned()).collect();
-        // let mut registry = self.registry(root_sources.into_iter())?;
+        Ok((workspace, registry))
+    }
+
+    pub fn get_dependency_graph(&self, roots: Vec<PackageId>) -> CargoResult<Graph> {
+        let (workspace, registry) = self.get_registry_from_workspace_members()?;
 
         let (packages, resolve) = our_resolve(
-            &mut registry,
+            registry,
             &workspace,
             &self.features_list,
             self.cargo_opts.all_features,
             self.cargo_opts.no_default_features,
-            self.cargo_opts.no_dev_dependencies,
+            self.cargo_opts.dev_dependencies()?,
         )?;
 
         let rustc = self.config.load_global_rustc(Some(&workspace))?;
         let host = rustc.host.to_string();
 
-        let target = self.cargo_opts.target.as_ref().map(|target| target.as_ref().unwrap_or(&host).as_str());
+        let target = self
+            .cargo_opts
+            .target
+            .as_ref()
+            .map(|target| target.as_ref().unwrap_or(&host).as_str());
 
         let cfgs = get_cfgs(&rustc, target)?;
         let graph = build_graph(&resolve, &packages, roots.into_iter(), target, &cfgs)?;
@@ -423,15 +528,15 @@ impl Repo {
             .map(|m| m.summary().package_id())
             .collect();
 
-        let mut registry = self.registry(roots.iter().map(|pkgid| pkgid.source_id()))?;
+        let registry = self.registry(roots.iter().map(|pkgid| pkgid.source_id()))?;
 
         let (package_set, _resolve) = our_resolve(
-            &mut registry,
+            registry,
             &workspace,
             &self.features_list,
             self.cargo_opts.all_features,
             self.cargo_opts.no_default_features,
-            self.cargo_opts.no_dev_dependencies,
+            self.cargo_opts.dev_dependencies()?,
         )?;
         let mut source = self.load_source()?;
 
@@ -464,15 +569,15 @@ impl Repo {
             .map(|m| m.summary().package_id())
             .collect();
 
-        let mut registry = self.registry(roots.iter().map(|pkgid| pkgid.source_id()))?;
+        let registry = self.registry(roots.iter().map(|pkgid| pkgid.source_id()))?;
 
         let (package_set, _resolve) = our_resolve(
-            &mut registry,
+            registry,
             &workspace,
             &self.features_list,
             self.cargo_opts.all_features,
             self.cargo_opts.no_default_features,
-            self.cargo_opts.no_dev_dependencies,
+            self.cargo_opts.dev_dependencies()?,
         )?;
 
         for pkg_id in package_set.package_ids() {
@@ -502,17 +607,15 @@ impl Repo {
     */
 
     pub fn get_package_set(&self) -> Result<(PackageSet<'_>, Resolve)> {
-        let workspace = self.workspace()?;
-
-        let mut registry = self.registry(vec![].into_iter())?;
+        let (workspace, registry) = self.get_registry_from_workspace_members()?;
 
         our_resolve(
-            &mut registry,
+            registry,
             &workspace,
             &self.features_list,
             self.cargo_opts.all_features,
             self.cargo_opts.no_default_features,
-            self.cargo_opts.no_dev_dependencies,
+            self.cargo_opts.dev_dependencies()?,
         )
     }
 
@@ -596,8 +699,8 @@ impl Repo {
     ) -> Result<PackageId> {
         if unrelated {
             Ok(
-                    self.find_independent_pkg_id_by_selector(name, version)?
-                        .ok_or_else(|| format_err!("Could not find requested crate. Try updating cargo's registry index cache?"))?
+                self.find_independent_pkg_id_by_selector(name, version)?
+                    .ok_or_else(|| format_err!("Could not find requested crate. Try updating cargo's registry index cache?"))?
                 )
         } else {
             Ok(self.find_dependency_pkg_id_by_selector(name, version)?
@@ -626,3 +729,179 @@ impl Repo {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use cargo::util::config::Definition;
+
+    #[test]
+    fn test_prune_directory_source_replacement() {
+        // Test that:
+        // {
+        //     "source": {"crates-io": {"replace-with": my-vendor-source (from --config cli option)},
+        //                "another-source": {"registry": path/to/registry (from --config cli option)},
+        //                "my-vendor-source": {"directory": vendor (from --config cli option)}},
+        // }
+        // becomes:
+        // {
+        //     "source": {"another-source": {"registry": path/to/registry (from --config cli option)}},
+        // }
+        //
+        // the "my-vendor-source" should get removed because it's a directory source replacement,
+        // and "crates-io" should get removed because it referenced the removed "my-vendor-source"
+        let crates_io_source_replacement = ConfigValue::Table(
+            [(
+                "replace-with".into(),
+                ConfigValue::String("my-vendor-source".into(), Definition::Cli),
+            )]
+            .iter()
+            .cloned()
+            .collect(),
+            Definition::Cli,
+        );
+
+        let directory_replacement = ConfigValue::Table(
+            [(
+                "directory".into(),
+                ConfigValue::String("vendor".into(), Definition::Cli),
+            )]
+            .iter()
+            .cloned()
+            .collect(),
+            Definition::Cli,
+        );
+
+        let registry_replacement = ConfigValue::Table(
+            [(
+                "registry".into(),
+                ConfigValue::String("path/to/registry".into(), Definition::Cli),
+            )]
+            .iter()
+            .cloned()
+            .collect(),
+            Definition::Cli,
+        );
+
+        let source_table = ConfigValue::Table(
+            [
+                ("crates-io".into(), crates_io_source_replacement),
+                ("my-vendor-source".into(), directory_replacement),
+                ("another-source".into(), registry_replacement.clone()),
+            ]
+            .iter()
+            .cloned()
+            .collect(),
+            Definition::Cli,
+        );
+
+        let mut config_table = [("source".into(), source_table)].iter().cloned().collect();
+
+        let expected_source_table = ConfigValue::Table(
+            [("another-source".into(), registry_replacement)]
+                .iter()
+                .cloned()
+                .collect(),
+            Definition::Cli,
+        );
+        let expected_config_table = [("source".into(), expected_source_table)]
+            .iter()
+            .cloned()
+            .collect();
+
+        prune_directory_source_replacements(&mut config_table).unwrap();
+        assert_eq!(config_table, expected_config_table);
+    }
+
+    #[test]
+    fn test_prune_directory_source_replacement_nested() {
+        // Test that:
+        // {
+        //     "source": {"another-source": {"registry": path/to/registry (from --config cli option)},
+        //                "nested-vendor-source": {"directory": vendor (from --config cli option)},
+        //                "my-vendor-source": {"replace-with": nested-vendor-source (from --config cli option)},
+        //                "crates-io": {"replace-with": my-vendor-source (from --config cli option)}},
+        // }
+        // becomes:
+        // {
+        //     "source": {"another-source": {"registry": path/to/registry (from --config cli option)}},
+        // }
+        //
+        // the "nested-vendor-source" should get removed because it's a directory source replacement,
+        // and "my-vendor-source" should get removed because it referenced the removed "nested-vendor-source"
+        // and "crates-io" should get removed because it referenced the removed "my-vendor-source"
+        let crates_io_source_replacement = ConfigValue::Table(
+            [(
+                "replace-with".into(),
+                ConfigValue::String("my-vendor-source".into(), Definition::Cli),
+            )]
+            .iter()
+            .cloned()
+            .collect(),
+            Definition::Cli,
+        );
+
+        let nested_replacement = ConfigValue::Table(
+            [(
+                "replace-with".into(),
+                ConfigValue::String("nested-vendor-source".into(), Definition::Cli),
+            )]
+            .iter()
+            .cloned()
+            .collect(),
+            Definition::Cli,
+        );
+
+        let directory_replacement = ConfigValue::Table(
+            [(
+                "directory".into(),
+                ConfigValue::String("vendor".into(), Definition::Cli),
+            )]
+            .iter()
+            .cloned()
+            .collect(),
+            Definition::Cli,
+        );
+
+        let registry_replacement = ConfigValue::Table(
+            [(
+                "registry".into(),
+                ConfigValue::String("path/to/registry".into(), Definition::Cli),
+            )]
+            .iter()
+            .cloned()
+            .collect(),
+            Definition::Cli,
+        );
+
+        let source_table = ConfigValue::Table(
+            [
+                ("crates-io".into(), crates_io_source_replacement),
+                ("my-vendor-source".into(), nested_replacement),
+                ("nested-vendor-source".into(), directory_replacement),
+                ("another-source".into(), registry_replacement.clone()),
+            ]
+            .iter()
+            .cloned()
+            .collect(),
+            Definition::Cli,
+        );
+
+        let mut config_table = [("source".into(), source_table)].iter().cloned().collect();
+
+        let expected_source_table = ConfigValue::Table(
+            [("another-source".into(), registry_replacement)]
+                .iter()
+                .cloned()
+                .collect(),
+            Definition::Cli,
+        );
+        let expected_config_table = [("source".into(), expected_source_table)]
+            .iter()
+            .cloned()
+            .collect();
+
+        prune_directory_source_replacements(&mut config_table).unwrap();
+        assert_eq!(config_table, expected_config_table);
+    }
+}
