@@ -723,19 +723,19 @@ pub fn lookup_crates(query: &str, count: usize) -> Result<()> {
         proof_count: usize,
     }
 
-    use crates_io_api::{ListOptions, Sort, SyncClient};
+    use crates_io_api::{CratesQuery, Sort, SyncClient};
 
     let local = crev_lib::Local::auto_create_or_open()?;
     let db = local.load_db()?;
 
     let client = SyncClient::new("cargo-crev", std::time::Duration::from_secs(1))?;
     let mut stats: Vec<_> = client
-        .crates(ListOptions {
-            sort: Sort::Downloads,
-            per_page: 100,
-            page: 1,
-            query: Some(query.to_string()),
-        })?
+        .crates(CratesQuery::builder()
+            .sort(Sort::Downloads)
+            .page_size(100)
+            .search(query.to_string())
+            .build()
+        )?
         .crates
         .iter()
         .map(|crate_| CrateStats {
