@@ -30,8 +30,8 @@ pub fn known_owners_count_color(count: u64) -> Option<color::Color> {
 /// Helper to control (possibly) colored output
 pub struct Term {
     pub stdout_is_tty: bool,
-    pub stderr_is_tty: bool,
-    pub stdin_is_tty: bool,
+    stderr_is_tty: bool,
+    stdin_is_tty: bool,
     stdout: Option<Box<StdoutTerminal>>,
     #[allow(unused)]
     stderr: Option<Box<StderrTerminal>>,
@@ -89,7 +89,6 @@ impl Term {
         Ok(())
     }
 
-    #[allow(unused)]
     pub fn eprint<C>(&mut self, fmt: Arguments<'_>, color: C) -> io::Result<()>
     where
         C: Into<Option<Color>>,
@@ -106,6 +105,25 @@ impl Term {
         }
         Ok(())
     }
+
+    pub fn eprintln<C>(&mut self, fmt: Arguments<'_>, color: C) -> io::Result<()>
+    where
+        C: Into<Option<Color>>,
+    {
+        let color = color.into();
+        self.print(fmt, color)?;
+        self.print(format_args!("\n"), color)?;
+
+        Ok(())
+    }
+
+    pub(crate) fn is_interactive(&self) -> bool {
+        self.stderr_is_tty && self.stdout_is_tty
+    }
+
+    pub(crate) fn is_input_interactive(&self) -> bool {
+        self.stdin_is_tty
+    }
 }
 
 pub fn read_passphrase() -> io::Result<String> {
